@@ -74,7 +74,7 @@ function install_federation {
     fi
 }
 
-function upload_sp_metadata {
+function upload_sp_metadata_to_testshib {
     local metadata_fname=${HOST_IP//./}_"$RANDOM"_sp
     local metadata_url=http://$HOST_IP/Shibboleth.sso/Metadata
 
@@ -113,8 +113,11 @@ function configure_federation {
         restart_service "devstack@keystone"
     fi
 
-    # Register the service provider
-    upload_sp_metadata
+    # TODO(knikolla): We should not be relying on an external service. This
+    # will be removed once we have an idp deployed during devstack install.
+    if [[ "$IDP_ID" == "testshib" ]]; then
+        upload_sp_metadata_to_testshib
+    fi
 }
 
 function register_federation {
