@@ -24,7 +24,6 @@ from keystone import exception
 from keystone.federation import constants as federation_constants
 from keystone.federation import utils
 from keystone.i18n import _
-from keystone.models import token_model
 from keystone import notifications
 
 LOG = log.getLogger(__name__)
@@ -37,9 +36,7 @@ class Mapped(base.AuthMethodHandler):
 
     def _get_token_ref(self, auth_payload):
         token_id = auth_payload['id']
-        response = PROVIDERS.token_provider_api.validate_token(token_id)
-        return token_model.KeystoneToken(token_id=token_id,
-                                         token_data=response)
+        return PROVIDERS.token_provider_api.validate_token(token_id)
 
     def authenticate(self, request, auth_payload):
         """Authenticate mapped user and set an authentication context.
@@ -72,14 +69,16 @@ class Mapped(base.AuthMethodHandler):
                                         response_data=response_data)
 
 
-def handle_scoped_token(request, token_ref, federation_api, identity_api):
+def handle_scoped_token(request, token, federation_api, identity_api):
     response_data = {}
-    utils.validate_expiration(token_ref)
-    token_audit_id = token_ref.audit_id
-    identity_provider = token_ref.federation_idp_id
-    protocol = token_ref.federation_protocol_id
-    user_id = token_ref.user_id
-    group_ids = token_ref.federation_group_ids
+    utils.validate_expiration(token)
+    token_audit_id = token.audit_id
+    identity_provider = token.identity_provider_id
+    protocol = token.protocol_id
+    user_id = token.user_id
+    group_ids = []
+    for group_dict in token.federated_groups:
+        group_ids.append(group_dict['id'])
     send_notification = functools.partial(
         notifications.send_saml_audit_notification, 'authenticate',
         request, user_id, group_ids, identity_provider, protocol,
@@ -237,9 +236,11 @@ def handle_unscoped_token(request, auth_payload, resource_api, federation_api,
             unique_id, display_name = (
                 get_user_unique_id_and_display_name(request, mapped_properties)
             )
+            email = mapped_properties['user'].get('email')
             user = identity_api.shadow_federated_user(identity_provider,
                                                       protocol, unique_id,
-                                                      display_name)
+                                                      display_name,
+                                                      email)
 
             if 'projects' in mapped_properties:
                 idp_domain_id = federation_api.get_idp(
@@ -371,5 +372,7 @@ def get_user_unique_id_and_display_name(request, mapped_properties):
     elif not user_id:
         user_id = user_name
 
+    if user_name:
+        user['name'] = user_name
     user['id'] = parse.quote(user_id)
     return (user['id'], user['name'])
