@@ -17,31 +17,18 @@ from oslo_log import versionutils
 
 from keystone.conf import utils
 
-
 bind = cfg.ListOpt(
     'bind',
     default=[],
-    help=utils.fmt("""
-This is a list of external authentication mechanisms which should add token
-binding metadata to tokens, such as `kerberos` or `x509`. Binding metadata is
-enforced according to the `[token] enforce_token_bind` option.
-"""))
-
-enforce_token_bind = cfg.StrOpt(
-    'enforce_token_bind',
-    default='permissive',
     deprecated_since=versionutils.deprecated.PIKE,
     deprecated_for_removal=True,
     help=utils.fmt("""
-This controls the token binding enforcement policy on tokens presented to
-keystone with token binding metadata (as specified by the `[token] bind`
-option). `disabled` completely bypasses token binding validation. `permissive`
-and `strict` do not require tokens to have binding metadata (but will validate
-it if present), whereas `required` will always demand tokens to having binding
-metadata. `permissive` will allow unsupported binding metadata to pass through
-without validation (usually to be validated at another time by another
-component), whereas `strict` and `required` will demand that the included
-binding metadata be supported by keystone.
+This is a list of external authentication mechanisms which should add token
+binding metadata to tokens, such as `kerberos` or `x509`. Note that this option
+is deprecated as keystone no longer supports binding metadata to tokens
+directly. This option is silently ignored and will be removed in the future.
+This option no longer has any impact on the behavior of tokens and can be
+removed.
 """))
 
 expiration = cfg.IntOpt(
@@ -53,10 +40,10 @@ expiration = cfg.IntOpt(
 The amount of time that a token should remain valid (in seconds). Drastically
 reducing this value may break "long-running" operations that involve multiple
 services to coordinate together, and will force users to authenticate with
-keystone more frequently. Drastically increasing this value will increase load
-on the `[token] driver`, as more tokens will be simultaneously valid. Keystone
-tokens are also bearer tokens, so a shorter duration will also reduce the
-potential security impact of a compromised token.
+keystone more frequently. Drastically increasing this value will increase the
+number of tokens that will be simultaneously valid. Keystone tokens are also
+bearer tokens, so a shorter duration will also reduce the potential security
+impact of a compromised token.
 """))
 
 provider = cfg.StrOpt(
@@ -65,27 +52,12 @@ provider = cfg.StrOpt(
     help=utils.fmt("""
 Entry point for the token provider in the `keystone.token.provider` namespace.
 The token provider controls the token construction, validation, and revocation
-operations. Keystone includes `fernet` and `uuid` token
-providers. `uuid` tokens must be persisted (using the backend specified in the
-`[token] driver` option), but do not require any extra configuration or setup.
+operations. Keystone includes `fernet` token provider.
 `fernet` tokens do not need to be persisted at all, but require that you run
 `keystone-manage fernet_setup` (also see the `keystone-manage fernet_rotate`
 command).
 """))
 
-driver = cfg.StrOpt(
-    'driver',
-    default='sql',
-    deprecated_since=versionutils.deprecated.PIKE,
-    deprecated_for_removal=True,
-    help=utils.fmt("""
-Entry point for the token persistence backend driver in the
-`keystone.token.persistence` namespace. Keystone provides the `sql`
-driver.  The `sql` option (default) depends on the options in your
-`[database]` section. If you're using the `fernet` `[token] provider`, this
-backend will not be utilized to persist tokens at all.
-"""))
-
 caching = cfg.BoolOpt(
     'caching',
     default=True,
@@ -129,6 +101,14 @@ request for a scoped token to avoid re-scoping altogether.
 infer_roles = cfg.BoolOpt(
     'infer_roles',
     default=True,
+    deprecated_since=versionutils.deprecated.ROCKY,
+    deprecated_reason=utils.fmt("""
+Default roles depend on a chain of implied role assignments. Ex: an admin user
+will also have the reader and member role. By ensuring that all these roles
+will always appear on the token validation response, we can improve the
+simplicity and readability of policy files.
+"""),
+    deprecated_for_removal=True,
     help=utils.fmt("""
 This controls whether roles should be included with tokens that are not
 directly assigned to the token's scope, but are instead linked implicitly to
@@ -156,11 +136,8 @@ Defaults to two days.
 
 GROUP_NAME = __name__.split('.')[-1]
 ALL_OPTS = [
-    bind,
-    enforce_token_bind,
     expiration,
     provider,
-    driver,
     caching,
     cache_time,
     revoke_by_id,
