@@ -268,8 +268,11 @@ def validate_mapping_structure(ref):
         raise exception.ValidationError(messages)
 
 
-def validate_expiration(token_ref):
-    if timeutils.utcnow() > token_ref.expires:
+def validate_expiration(token):
+    token_expiration_datetime = timeutils.normalize_time(
+        timeutils.parse_isotime(token.expires_at)
+    )
+    if timeutils.utcnow() > token_expiration_datetime:
         raise exception.Unauthorized(_('Federation token is expired'))
 
 
@@ -409,8 +412,8 @@ def transform_to_group_ids(group_names, mapping_id,
                 group['name'], resolve_domain(group['domain']))
             yield group_dict['id']
         except exception.GroupNotFound:
-            LOG.debug('Group %s has no entry in the backend',
-                      group['name'])
+            raise exception.MappedGroupNotFound(group_id=group['name'],
+                                                mapping_id=mapping_id)
 
 
 def get_assertion_params_from_env(request):
