@@ -28,6 +28,7 @@ from keystone.common import provider_api
 from keystone.common.validation import validators
 from keystone import exception
 from keystone import middleware
+from keystone.resource.backends import base as resource_base
 from keystone.tests.common import auth as common_auth
 from keystone.tests import unit
 from keystone.tests.unit import rest
@@ -45,6 +46,20 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
     def generate_token_schema(self, system_scoped=False, domain_scoped=False,
                               project_scoped=False):
         """Return a dictionary of token properties to validate against."""
+        ROLES_SCHEMA = {
+            'type': 'array',
+            'items': {
+                'type': 'object',
+                'properties': {
+                    'id': {'type': 'string', },
+                    'name': {'type': 'string', },
+                },
+                'required': ['id', 'name', ],
+                'additionalProperties': False,
+            },
+            'minItems': 1,
+        }
+
         properties = {
             'audit_ids': {
                 'type': 'array',
@@ -110,34 +125,10 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                     'all': {'type': 'boolean'}
                 }
             }
-            properties['roles'] = {
-                'type': 'array',
-                'items': {
-                    'type': 'object',
-                    'properties': {
-                        'id': {'type': 'string', },
-                        'name': {'type': 'string', },
-                    },
-                    'required': ['id', 'name', ],
-                    'additionalProperties': False,
-                },
-                'minItems': 1,
-            }
+            properties['roles'] = ROLES_SCHEMA
         elif domain_scoped:
             properties['catalog'] = {'type': 'array'}
-            properties['roles'] = {
-                'type': 'array',
-                'items': {
-                    'type': 'object',
-                    'properties': {
-                        'id': {'type': 'string', },
-                        'name': {'type': 'string', },
-                    },
-                    'required': ['id', 'name', ],
-                    'additionalProperties': False,
-                },
-                'minItems': 1,
-            }
+            properties['roles'] = ROLES_SCHEMA
             properties['domain'] = {
                 'type': 'object',
                 'required': ['id', 'name'],
@@ -150,7 +141,12 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         elif project_scoped:
             properties['is_admin_project'] = {'type': 'boolean'}
             properties['catalog'] = {'type': 'array'}
-            properties['roles'] = {'type': 'array'}
+            # FIXME(lbragstad): Remove this in favor of the predefined
+            # ROLES_SCHEMA dictionary once bug 1763510 is fixed.
+            ROLES_SCHEMA['items']['properties']['domain_id'] = {
+                'type': ['null', 'string', ],
+            }
+            properties['roles'] = ROLES_SCHEMA
             properties['is_domain'] = {'type': 'boolean'}
             properties['project'] = {
                 'type': ['object'],
@@ -200,9 +196,11 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         config_files.append(unit.dirs.tests_conf('backend_sql.conf'))
         return config_files
 
-    def setUp(self, app_conf='keystone'):
+    def setUp(self, app_conf='keystone', enable_sqlite_foreign_key=False):
         """Setup for v3 Restful Test Cases."""
-        super(RestfulTestCase, self).setUp(app_conf=app_conf)
+        super(RestfulTestCase, self).setUp(
+            app_conf=app_conf,
+            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
 
         self.empty_context = {'environment': {}}
 
@@ -212,21 +210,37 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
         super(RestfulTestCase, self).load_backends()
 
-    def load_fixtures(self, fixtures):
-        self.load_sample_data()
+    def load_fixtures(self, fixtures, enable_sqlite_foreign_key=False):
+        self.load_sample_data(
+            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
 
-    def _populate_default_domain(self):
+    def _populate_default_domain(self, enable_sqlite_foreign_key=False):
         try:
             PROVIDERS.resource_api.get_domain(DEFAULT_DOMAIN_ID)
         except exception.DomainNotFound:
+            # TODO(wxy): when FK is enabled in sqlite, a lot of tests will fail
+            # due to the root domain is missing. So we should open FKs for the
+            # tests one by one. If the FKs is enabled for one test,
+            # `enable_sqlite_foreign_key` should be `true` here to ensure the
+            # root domain is created. Once all tests enable FKs, the
+            # ``enable_sqlite_foreign_key`` can be removed.
+            if enable_sqlite_foreign_key:
+                root_domain = unit.new_domain_ref(
+                    id=resource_base.NULL_DOMAIN_ID,
+                    name=resource_base.NULL_DOMAIN_ID
+                )
+                self.resource_api.create_domain(resource_base.NULL_DOMAIN_ID,
+                                                root_domain)
             domain = unit.new_domain_ref(
                 description=(u'The default domain'),
                 id=DEFAULT_DOMAIN_ID,
                 name=u'Default')
             PROVIDERS.resource_api.create_domain(DEFAULT_DOMAIN_ID, domain)
 
-    def load_sample_data(self, create_region_and_endpoints=True):
-        self._populate_default_domain()
+    def load_sample_data(self, create_region_and_endpoints=True,
+                         enable_sqlite_foreign_key=False):
+        self._populate_default_domain(
+            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
         self.domain = unit.new_domain_ref()
         self.domain_id = self.domain['id']
         PROVIDERS.resource_api.create_domain(self.domain_id, self.domain)
@@ -390,32 +404,6 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
             })
         return r.headers.get('X-Subject-Token')
 
-    def get_system_scoped_token(self):
-        """Convenience method for requesting system scoped tokens."""
-        r = self.admin_request(
-            method='POST',
-            path='/v3/auth/tokens',
-            body={
-                'auth': {
-                    'identity': {
-                        'methods': ['password'],
-                        'password': {
-                            'user': {
-                                'name': self.user['name'],
-                                'password': self.user['password'],
-                                'domain': {
-                                    'id': self.user['domain_id']
-                                }
-                            }
-                        }
-                    },
-                    'scope': {
-                        'system': {'all': True}
-                    }
-                }
-            })
-        return r.headers.get('X-Subject-Token')
-
     def get_domain_scoped_token(self):
         """Convenience method for requesting domain scoped token."""
         r = self.admin_request(
@@ -753,31 +741,6 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
         return token
 
-    def assertEqualTokens(self, a, b):
-        """Assert that two tokens are equal.
-
-        Compare two tokens except for their ids. This also truncates
-        the time in the comparison.
-        """
-        def normalize(token):
-            del token['token']['expires_at']
-            del token['token']['issued_at']
-            return token
-
-        a_expires_at = self.assertValidISO8601ExtendedFormatDatetime(
-            a['token']['expires_at'])
-        b_expires_at = self.assertValidISO8601ExtendedFormatDatetime(
-            b['token']['expires_at'])
-        self.assertCloseEnoughForGovernmentWork(a_expires_at, b_expires_at)
-
-        a_issued_at = self.assertValidISO8601ExtendedFormatDatetime(
-            a['token']['issued_at'])
-        b_issued_at = self.assertValidISO8601ExtendedFormatDatetime(
-            b['token']['issued_at'])
-        self.assertCloseEnoughForGovernmentWork(a_issued_at, b_issued_at)
-
-        return self.assertDictEqual(normalize(a), normalize(b))
-
     # catalog validation
 
     def assertValidCatalogResponse(self, resp, *args, **kwargs):
@@ -1230,6 +1193,8 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                 self.assertValidRole(role)
 
             self.assertValidListLinks(entity.get('roles_links'))
+            # Mirror the test tempest does to ensure the self-link is correct
+            self.assertIn('v3/OS-TRUST/trusts', entity.get('links')['self'])
 
             # always disallow role xor project_id (neither or both is allowed)
             has_roles = bool(entity.get('roles'))
@@ -1261,31 +1226,24 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         for attribute in attributes:
             self.assertIsNotNone(entity.get(attribute))
 
-    def assertValidServiceProviderListResponse(self, resp, *args, **kwargs):
-        if kwargs.get('keys_to_check') is None:
-            kwargs['keys_to_check'] = ['auth_url', 'id', 'enabled',
-                                       'description', 'relay_state_prefix',
-                                       'sp_url']
-        return self.assertValidListResponse(
-            resp,
-            'service_providers',
-            self.assertValidServiceProvider,
-            *args,
-            **kwargs)
-
     def build_external_auth_request(self, remote_user,
                                     remote_domain=None, auth_data=None,
                                     kerberos=False):
-        environment = {'REMOTE_USER': remote_user, 'AUTH_TYPE': 'Negotiate'}
-        if remote_domain:
-            environment['REMOTE_DOMAIN'] = remote_domain
+        environment = self.build_external_auth_environ(
+            remote_user, remote_domain)
         if not auth_data:
             auth_data = self.build_authentication_request(
                 kerberos=kerberos)['auth']
         auth_info = auth.core.AuthInfo.create(auth_data)
-        auth_context = auth.core.AuthContext(extras={}, method_names=[])
+        auth_context = auth.core.AuthContext(method_names=[])
         return self.make_request(environ=environment), auth_info, auth_context
 
+    def build_external_auth_environ(self, remote_user, remote_domain=None):
+        environment = {'REMOTE_USER': remote_user, 'AUTH_TYPE': 'Negotiate'}
+        if remote_domain:
+            environment['REMOTE_DOMAIN'] = remote_domain
+        return environment
+
 
 class VersionTestCase(RestfulTestCase):
     def test_get_version(self):
